"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetch = exports.getCredentialedURL = exports.getURL = exports.removeHeaders = exports.getHeaders = exports.getAuthHeader = exports.getQueryString = exports.redactSearchParams = exports.removeSearchParams = exports.getSearchParams = exports.getAccessToken = exports.validate = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
// defaults
const process_1 = require("process");
const envCredentials = process_1.env;
/**
 * Check whether or not sufficient GitHub credentials were supplied.
 * @returns `true` if valid
 * @throws if invalid
 */
function validate(credentials) {
    const accessToken = getAccessToken(credentials);
    if (accessToken ||
        (credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET)) {
        return true;
    }
    else {
        throw new Error('missing github credentials; provide `GITHUB_ACCESS_TOKEN` or `GITHUB_TOKEN`, or a combination of `GITHUB_CLIENT_ID` and `GITHUB_CLIENT_SECRET`');
    }
}
exports.validate = validate;
/**
 * Get the desired GitHub Access Token from the credentials.
 * You probably want to use {@link fetch} directly, instead of going through this method.
 */
function getAccessToken(credentials) {
    return credentials.GITHUB_ACCESS_TOKEN || credentials.GITHUB_TOKEN || null;
}
exports.getAccessToken = getAccessToken;
/**
 * Get the GitHub Authorization Search Params.
 * You probably want to use {@link fetch} directly, instead of going through this method.
 * @param credentials The params to use for the authorization variables.
 * @param params If you wish to set the params on an existing URLSearchParams instance, then provide it here.
 * @returns The search params that you should append to your github API request url.
 * @throws If no valid GitHub Authorization was provided.
 */
function getSearchParams(credentials, params = new URLSearchParams()) {
    const accessToken = getAccessToken(credentials);
    if (accessToken) {
        params.set('access_token', accessToken);
    }
    else if (credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET) {
        params.set('client_id', credentials.GITHUB_CLIENT_ID);
        params.set('client_secret', credentials.GITHUB_CLIENT_SECRET);
    }
    else {
        // throw with detail errors
        validate(credentials);
        // if that doesn't throw, then fallback to this
        throw new Error('invalid github credentials');
    }
    return params;
}
exports.getSearchParams = getSearchParams;
/**
 * Remove any GitHub Credentials from a URL Search Params instance.
 * You probably want to use {@link fetch} directly, instead of going through this method.
 */
function removeSearchParams(params = new URLSearchParams()) {
    params.delete('access_token');
    params.delete('client_id');
    params.delete('client_secret');
    return params;
}
exports.removeSearchParams = removeSearchParams;
/**
 * Redact any GitHub Credentials from a URL string.
 * You probably want to use {@link fetch} directly, instead of going through this method.
 * @param value The string to redact credentials from.
 * @returns The string with the credentials redacted.
 */
function redactSearchParams(value) {
    return value.replace(/(&?)(access_token|client_id|client_secret)=\w+/gi, '$1$2=REDACTED');
}
exports.redactSearchParams = redactSearchParams;
/**
 * Get the GitHub Authorization as a Query String.
 * You probably want to use {@link getURL} directly, instead of going through this method.
 */
function getQueryString(credentials) {
    return getSearchParams(credentials).toString();
}
exports.getQueryString = getQueryString;
/**
 * Get the GitHub Authorization Header.
 * Use as the `Authorization` header within {@link fetch} calls.
 * You probably want to use {@link getHeaders} or {@link fetch} directly, instead of going through this method.
 * @throws If no valid GitHub Authorization was provided.
 */
function getAuthHeader(credentials) {
    const accessToken = getAccessToken(credentials);
    if (accessToken) {
        return `token ${accessToken}`;
    }
    else if (credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET) {
        return `Basic ${credentials.GITHUB_CLIENT_ID}:${credentials.GITHUB_CLIENT_SECRET}`;
    }
    else {
        throw new Error('missing github credentials for authorization header');
    }
}
exports.getAuthHeader = getAuthHeader;
/**
 * Get the headers to attach to the request to the GitHub API.
 * Use as the headers object within {@link fetch} calls.
 * You probably want to use {@link fetch} directly, instead of going through this method.
 */
function getHeaders(credentials, headers = {}) {
    return {
        Accept: 'application/vnd.github.v3+json',
        Authorization: getAuthHeader(credentials),
        ...headers,
    };
}
exports.getHeaders = getHeaders;
/**
 * Remove any GitHub Credentials from a Headers instance.
 * You probably want to use {@link fetch} directly, instead of going through this method.
 */
function removeHeaders(headers) {
    // @ts-ignore
    delete headers.Authorization;
    return headers;
}
exports.removeHeaders = removeHeaders;
/**
 * Get the desired Github API URL, using {@link removeSearchParams}.to ensure there are no credentials.
 * As this URL does not include credentials, use with {@link getAuthHeader} to authorize correctly.
 * Otherwise use {@link getCredentialedURL} to get a credentialed URL.
 * You probably want to use {@link fetch} directly, instead of going through this method.
 * If the credentials property is nullish, then the environment variables are attempted.
 */
function getURL(props = {}) {
    // default credentials
    if (props.credentials == null)
        props = { ...props, credentials: envCredentials };
    // fetch url
    const url = new URL(props.url ||
        props.credentials.GITHUB_API_URL ||
        props.credentials.GITHUB_API ||
        'https://api.github.com');
    // add user params
    if (props === null || props === void 0 ? void 0 : props.searchParams) {
        if (props.searchParams instanceof URLSearchParams) {
            props.searchParams.forEach((value, key) => url.searchParams.set(key, value));
        }
        else {
            Object.entries(props.searchParams).forEach(([key, value]) => url.searchParams.set(key, value));
        }
    }
    // ensure that there are no credentials in the URL
    removeSearchParams(url.searchParams);
    // add user pathname
    // the convoluted way of doing this is to make sure that with or without / is valid
    // as the GITHUB_API hostname may be something like `https://bevry.me/api/github`
    if (props === null || props === void 0 ? void 0 : props.pathname) {
        url.pathname = [
            url.pathname.replace(/^[/]+|[/]+$/, ''),
            props.pathname.replace(/^[/]+|[/]+$/, ''),
        ]
            .filter((i) => i)
            .join('/');
    }
    // return
    return url;
}
exports.getURL = getURL;
/**
 * Get the credentialed GitHub API URL instance.
 * Uses {@link getURL} to get the URL, then uses {@link getSearchParams} to add the credentials.
 * You probably want to use {@link fetch} directly, instead of going through this method.
 * If the credentials property is nullish, then the environment variables are attempted.
 */
function getCredentialedURL(props = {}) {
    // default credentials
    if (props.credentials == null)
        props = { ...props, credentials: envCredentials };
    // fetch url
    const url = getURL(props);
    // add auth params
    getSearchParams(props.credentials, url.searchParams);
    // return
    return url;
}
exports.getCredentialedURL = getCredentialedURL;
/**
 * Fetches a GitHub API response via secure headers authorization.
 * Uses {@link getURL} to get the URL, then uses {@link getHeaders} to add the credentials.
 * This is probably the method you want to use.
 * If the credentials property is nullish, then the environment variables are attempted.
 */
function fetch(props = {}) {
    // default credentials
    if (props.credentials == null)
        props = { ...props, credentials: envCredentials };
    // prep
    const url = getURL(props);
    const opts = {
        headers: getHeaders(props.credentials, props.headers),
    };
    // fetch and return
    return node_fetch_1.default(url, opts);
}
exports.fetch = fetch;
